
# qhasm: int64 rp

# qhasm: int64 pp

# qhasm: int64 qp

# qhasm: input rp

# qhasm: input pp

# qhasm: input qp

# qhasm:   int64 caller1

# qhasm:   int64 caller2

# qhasm:   int64 caller3

# qhasm:   int64 caller4

# qhasm:   int64 caller5

# qhasm:   int64 caller6

# qhasm:   int64 caller7

# qhasm:   caller caller1

# qhasm:   caller caller2

# qhasm:   caller caller3

# qhasm:   caller caller4

# qhasm:   caller caller5

# qhasm:   caller caller6

# qhasm:   caller caller7

# qhasm:   stack64 caller1_stack

# qhasm:   stack64 caller2_stack

# qhasm:   stack64 caller3_stack

# qhasm:   stack64 caller4_stack

# qhasm:   stack64 caller5_stack

# qhasm:   stack64 caller6_stack

# qhasm:   stack64 caller7_stack

# qhasm: int64 a0

# qhasm: int64 a1

# qhasm: int64 a2

# qhasm: int64 a3

# qhasm: int64 a4

# qhasm: stack64 a0_stack

# qhasm: stack64 a1_stack

# qhasm: stack64 a2_stack

# qhasm: stack64 a3_stack

# qhasm: stack64 a4_stack

# qhasm: int64 b0

# qhasm: int64 b1

# qhasm: int64 b2

# qhasm: int64 b3

# qhasm: int64 b4

# qhasm: stack64 b0_stack

# qhasm: stack64 b1_stack

# qhasm: stack64 b2_stack

# qhasm: stack64 b3_stack

# qhasm: stack64 b4_stack

# qhasm: int64 c0

# qhasm: int64 c1

# qhasm: int64 c2

# qhasm: int64 c3

# qhasm: int64 c4

# qhasm: stack64 c0_stack

# qhasm: stack64 c1_stack

# qhasm: stack64 c2_stack

# qhasm: stack64 c3_stack

# qhasm: stack64 c4_stack

# qhasm: int64 d0

# qhasm: int64 d1

# qhasm: int64 d2

# qhasm: int64 d3

# qhasm: int64 d4

# qhasm: stack64 d0_stack

# qhasm: stack64 d1_stack

# qhasm: stack64 d2_stack

# qhasm: stack64 d3_stack

# qhasm: stack64 d4_stack

# qhasm: int64 e0

# qhasm: int64 e1

# qhasm: int64 e2

# qhasm: int64 e3

# qhasm: int64 e4

# qhasm: stack64 e0_stack

# qhasm: stack64 e1_stack

# qhasm: stack64 e2_stack

# qhasm: stack64 e3_stack

# qhasm: stack64 e4_stack

# qhasm: int64 f0

# qhasm: int64 f1

# qhasm: int64 f2

# qhasm: int64 f3

# qhasm: int64 f4

# qhasm: stack64 f0_stack

# qhasm: stack64 f1_stack

# qhasm: stack64 f2_stack

# qhasm: stack64 f3_stack

# qhasm: stack64 f4_stack

# qhasm: int64 g0

# qhasm: int64 g1

# qhasm: int64 g2

# qhasm: int64 g3

# qhasm: int64 g4

# qhasm: stack64 g0_stack

# qhasm: stack64 g1_stack

# qhasm: stack64 g2_stack

# qhasm: stack64 g3_stack

# qhasm: stack64 g4_stack

# qhasm: int64 h0

# qhasm: int64 h1

# qhasm: int64 h2

# qhasm: int64 h3

# qhasm: int64 h4

# qhasm: stack64 h0_stack

# qhasm: stack64 h1_stack

# qhasm: stack64 h2_stack

# qhasm: stack64 h3_stack

# qhasm: stack64 h4_stack

# qhasm: int64 qt0

# qhasm: int64 qt1

# qhasm: int64 qt2

# qhasm: int64 qt3

# qhasm: int64 qt4

# qhasm: stack64 qt0_stack

# qhasm: stack64 qt1_stack

# qhasm: stack64 qt2_stack

# qhasm: stack64 qt3_stack

# qhasm: stack64 qt4_stack

# qhasm: int64 t10

# qhasm: int64 t11

# qhasm: int64 t12

# qhasm: int64 t13

# qhasm: int64 t14

# qhasm: stack64 t10_stack

# qhasm: stack64 t11_stack

# qhasm: stack64 t12_stack

# qhasm: stack64 t13_stack

# qhasm: stack64 t14_stack

# qhasm: int64 t20

# qhasm: int64 t21

# qhasm: int64 t22

# qhasm: int64 t23

# qhasm: int64 t24

# qhasm: stack64 t20_stack

# qhasm: stack64 t21_stack

# qhasm: stack64 t22_stack

# qhasm: stack64 t23_stack

# qhasm: stack64 t24_stack

# qhasm: int64 rx0

# qhasm: int64 rx1

# qhasm: int64 rx2

# qhasm: int64 rx3

# qhasm: int64 rx4

# qhasm: int64 ry0

# qhasm: int64 ry1

# qhasm: int64 ry2

# qhasm: int64 ry3

# qhasm: int64 ry4

# qhasm: int64 rz0

# qhasm: int64 rz1

# qhasm: int64 rz2

# qhasm: int64 rz3

# qhasm: int64 rz4

# qhasm: int64 rt0

# qhasm: int64 rt1

# qhasm: int64 rt2

# qhasm: int64 rt3

# qhasm: int64 rt4

# qhasm: int64 mulr01

# qhasm: int64 mulr11

# qhasm: int64 mulr21

# qhasm: int64 mulr31

# qhasm: int64 mulr41

# qhasm: int64 mulrax

# qhasm: int64 mulrdx

# qhasm: int64 mult

# qhasm: int64 mulredmask

# qhasm: stack64 mulx219_stack

# qhasm: stack64 mulx319_stack

# qhasm: stack64 mulx419_stack

# qhasm: enter crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd_p1p1
.text
.p2align 5
.globl _crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd_p1p1
.globl crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd_p1p1
_crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd_p1p1:
crypto_sign_ed25519_amd64_51_30k_batch_ge25519_nielsadd_p1p1:
mov %rsp,%r11
and $31,%r11
add $160,%r11
sub %r11,%rsp

# qhasm:   caller1_stack = caller1
# asm 1: movq <caller1=int64#9,>caller1_stack=stack64#1
# asm 2: movq <caller1=%r11,>caller1_stack=0(%rsp)
movq %r11,0(%rsp)

# qhasm:   caller2_stack = caller2
# asm 1: movq <caller2=int64#10,>caller2_stack=stack64#2
# asm 2: movq <caller2=%r12,>caller2_stack=8(%rsp)
movq %r12,8(%rsp)

# qhasm:   caller3_stack = caller3
# asm 1: movq <caller3=int64#11,>caller3_stack=stack64#3
# asm 2: movq <caller3=%r13,>caller3_stack=16(%rsp)
movq %r13,16(%rsp)

# qhasm:   caller4_stack = caller4
# asm 1: movq <caller4=int64#12,>caller4_stack=stack64#4
# asm 2: movq <caller4=%r14,>caller4_stack=24(%rsp)
movq %r14,24(%rsp)

# qhasm:   caller5_stack = caller5
# asm 1: movq <caller5=int64#13,>caller5_stack=stack64#5
# asm 2: movq <caller5=%r15,>caller5_stack=32(%rsp)
movq %r15,32(%rsp)

# qhasm:   caller6_stack = caller6
# asm 1: movq <caller6=int64#14,>caller6_stack=stack64#6
# asm 2: movq <caller6=%rbx,>caller6_stack=40(%rsp)
movq %rbx,40(%rsp)

# qhasm:   caller7_stack = caller7
# asm 1: movq <caller7=int64#15,>caller7_stack=stack64#7
# asm 2: movq <caller7=%rbp,>caller7_stack=48(%rsp)
movq %rbp,48(%rsp)

# qhasm: qp = qp
# asm 1: mov  <qp=int64#3,>qp=int64#4
# asm 2: mov  <qp=%rdx,>qp=%rcx
mov  %rdx,%rcx

# qhasm: a0 = *(uint64 *)(pp + 40)
# asm 1: movq   40(<pp=int64#2),>a0=int64#3
# asm 2: movq   40(<pp=%rsi),>a0=%rdx
movq   40(%rsi),%rdx

# qhasm: a1 = *(uint64 *)(pp + 48)
# asm 1: movq   48(<pp=int64#2),>a1=int64#5
# asm 2: movq   48(<pp=%rsi),>a1=%r8
movq   48(%rsi),%r8

# qhasm: a2 = *(uint64 *)(pp + 56)
# asm 1: movq   56(<pp=int64#2),>a2=int64#6
# asm 2: movq   56(<pp=%rsi),>a2=%r9
movq   56(%rsi),%r9

# qhasm: a3 = *(uint64 *)(pp + 64)
# asm 1: movq   64(<pp=int64#2),>a3=int64#7
# asm 2: movq   64(<pp=%rsi),>a3=%rax
movq   64(%rsi),%rax

# qhasm: a4 = *(uint64 *)(pp + 72)
# asm 1: movq   72(<pp=int64#2),>a4=int64#8
# asm 2: movq   72(<pp=%rsi),>a4=%r10
movq   72(%rsi),%r10

# qhasm: b0 = a0
# asm 1: mov  <a0=int64#3,>b0=int64#9
# asm 2: mov  <a0=%rdx,>b0=%r11
mov  %rdx,%r11

# qhasm: b1 = a1
# asm 1: mov  <a1=int64#5,>b1=int64#10
# asm 2: mov  <a1=%r8,>b1=%r12
mov  %r8,%r12

# qhasm: b2 = a2
# asm 1: mov  <a2=int64#6,>b2=int64#11
# asm 2: mov  <a2=%r9,>b2=%r13
mov  %r9,%r13

# qhasm: b3 = a3
# asm 1: mov  <a3=int64#7,>b3=int64#12
# asm 2: mov  <a3=%rax,>b3=%r14
mov  %rax,%r14

# qhasm: b4 = a4
# asm 1: mov  <a4=int64#8,>b4=int64#13
# asm 2: mov  <a4=%r10,>b4=%r15
mov  %r10,%r15

# qhasm: a0 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P0
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<a0=int64#3
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<a0=%rdx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P0(%rip),%rdx

# qhasm: a1 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a1=int64#5
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a1=%r8
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r8

# qhasm: a2 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a2=int64#6
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a2=%r9
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r9

# qhasm: a3 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a3=int64#7
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a3=%rax
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rax

# qhasm: a4 += *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a4=int64#8
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<a4=%r10
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r10

# qhasm: b0 += *(uint64 *) (pp + 0)
# asm 1: addq 0(<pp=int64#2),<b0=int64#9
# asm 2: addq 0(<pp=%rsi),<b0=%r11
addq 0(%rsi),%r11

# qhasm: b1 += *(uint64 *) (pp + 8)
# asm 1: addq 8(<pp=int64#2),<b1=int64#10
# asm 2: addq 8(<pp=%rsi),<b1=%r12
addq 8(%rsi),%r12

# qhasm: b2 += *(uint64 *) (pp + 16)
# asm 1: addq 16(<pp=int64#2),<b2=int64#11
# asm 2: addq 16(<pp=%rsi),<b2=%r13
addq 16(%rsi),%r13

# qhasm: b3 += *(uint64 *) (pp + 24)
# asm 1: addq 24(<pp=int64#2),<b3=int64#12
# asm 2: addq 24(<pp=%rsi),<b3=%r14
addq 24(%rsi),%r14

# qhasm: b4 += *(uint64 *) (pp + 32)
# asm 1: addq 32(<pp=int64#2),<b4=int64#13
# asm 2: addq 32(<pp=%rsi),<b4=%r15
addq 32(%rsi),%r15

# qhasm: a0 -= *(uint64 *) (pp + 0)
# asm 1: subq 0(<pp=int64#2),<a0=int64#3
# asm 2: subq 0(<pp=%rsi),<a0=%rdx
subq 0(%rsi),%rdx

# qhasm: a1 -= *(uint64 *) (pp + 8)
# asm 1: subq 8(<pp=int64#2),<a1=int64#5
# asm 2: subq 8(<pp=%rsi),<a1=%r8
subq 8(%rsi),%r8

# qhasm: a2 -= *(uint64 *) (pp + 16)
# asm 1: subq 16(<pp=int64#2),<a2=int64#6
# asm 2: subq 16(<pp=%rsi),<a2=%r9
subq 16(%rsi),%r9

# qhasm: a3 -= *(uint64 *) (pp + 24)
# asm 1: subq 24(<pp=int64#2),<a3=int64#7
# asm 2: subq 24(<pp=%rsi),<a3=%rax
subq 24(%rsi),%rax

# qhasm: a4 -= *(uint64 *) (pp + 32)
# asm 1: subq 32(<pp=int64#2),<a4=int64#8
# asm 2: subq 32(<pp=%rsi),<a4=%r10
subq 32(%rsi),%r10

# qhasm: a0_stack = a0
# asm 1: movq <a0=int64#3,>a0_stack=stack64#8
# asm 2: movq <a0=%rdx,>a0_stack=56(%rsp)
movq %rdx,56(%rsp)

# qhasm: a1_stack = a1
# asm 1: movq <a1=int64#5,>a1_stack=stack64#9
# asm 2: movq <a1=%r8,>a1_stack=64(%rsp)
movq %r8,64(%rsp)

# qhasm: a2_stack = a2
# asm 1: movq <a2=int64#6,>a2_stack=stack64#10
# asm 2: movq <a2=%r9,>a2_stack=72(%rsp)
movq %r9,72(%rsp)

# qhasm: a3_stack = a3
# asm 1: movq <a3=int64#7,>a3_stack=stack64#11
# asm 2: movq <a3=%rax,>a3_stack=80(%rsp)
movq %rax,80(%rsp)

# qhasm: a4_stack = a4
# asm 1: movq <a4=int64#8,>a4_stack=stack64#12
# asm 2: movq <a4=%r10,>a4_stack=88(%rsp)
movq %r10,88(%rsp)

# qhasm: b0_stack = b0
# asm 1: movq <b0=int64#9,>b0_stack=stack64#13
# asm 2: movq <b0=%r11,>b0_stack=96(%rsp)
movq %r11,96(%rsp)

# qhasm: b1_stack = b1
# asm 1: movq <b1=int64#10,>b1_stack=stack64#14
# asm 2: movq <b1=%r12,>b1_stack=104(%rsp)
movq %r12,104(%rsp)

# qhasm: b2_stack = b2
# asm 1: movq <b2=int64#11,>b2_stack=stack64#15
# asm 2: movq <b2=%r13,>b2_stack=112(%rsp)
movq %r13,112(%rsp)

# qhasm: b3_stack = b3
# asm 1: movq <b3=int64#12,>b3_stack=stack64#16
# asm 2: movq <b3=%r14,>b3_stack=120(%rsp)
movq %r14,120(%rsp)

# qhasm: b4_stack = b4
# asm 1: movq <b4=int64#13,>b4_stack=stack64#17
# asm 2: movq <b4=%r15,>b4_stack=128(%rsp)
movq %r15,128(%rsp)

# qhasm:   mulrax = a3_stack
# asm 1: movq <a3_stack=stack64#11,>mulrax=int64#3
# asm 2: movq <a3_stack=80(%rsp),>mulrax=%rdx
movq 80(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#18
# asm 2: movq <mulrax=%rax,>mulx319_stack=136(%rsp)
movq %rax,136(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#4)
# asm 2: mulq  16(<qp=%rcx)
mulq  16(%rcx)

# qhasm:   a0 = mulrax
# asm 1: mov  <mulrax=int64#7,>a0=int64#5
# asm 2: mov  <mulrax=%rax,>a0=%r8
mov  %rax,%r8

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#6
# asm 2: mov  <mulrdx=%rdx,>mulr01=%r9
mov  %rdx,%r9

# qhasm:   mulrax = a4_stack
# asm 1: movq <a4_stack=stack64#12,>mulrax=int64#3
# asm 2: movq <a4_stack=88(%rsp),>mulrax=%rdx
movq 88(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#19
# asm 2: movq <mulrax=%rax,>mulx419_stack=144(%rsp)
movq %rax,144(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#4)
# asm 2: mulq  8(<qp=%rcx)
mulq  8(%rcx)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#5
# asm 2: add  <mulrax=%rax,<a0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#4)
# asm 2: mulq  0(<qp=%rcx)
mulq  0(%rcx)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#5
# asm 2: add  <mulrax=%rax,<a0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#4)
# asm 2: mulq  8(<qp=%rcx)
mulq  8(%rcx)

# qhasm:   a1 = mulrax
# asm 1: mov  <mulrax=int64#7,>a1=int64#8
# asm 2: mov  <mulrax=%rax,>a1=%r10
mov  %rax,%r10

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#9
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r11
mov  %rdx,%r11

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#4)
# asm 2: mulq  16(<qp=%rcx)
mulq  16(%rcx)

# qhasm:   a2 = mulrax
# asm 1: mov  <mulrax=int64#7,>a2=int64#10
# asm 2: mov  <mulrax=%rax,>a2=%r12
mov  %rax,%r12

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#11
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r13
mov  %rdx,%r13

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#4)
# asm 2: mulq  24(<qp=%rcx)
mulq  24(%rcx)

# qhasm:   a3 = mulrax
# asm 1: mov  <mulrax=int64#7,>a3=int64#12
# asm 2: mov  <mulrax=%rax,>a3=%r14
mov  %rax,%r14

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#13
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r15
mov  %rdx,%r15

# qhasm:   mulrax = a0_stack
# asm 1: movq <a0_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <a0_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#4)
# asm 2: mulq  32(<qp=%rcx)
mulq  32(%rcx)

# qhasm:   a4 = mulrax
# asm 1: mov  <mulrax=int64#7,>a4=int64#14
# asm 2: mov  <mulrax=%rax,>a4=%rbx
mov  %rax,%rbx

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#15
# asm 2: mov  <mulrdx=%rdx,>mulr41=%rbp
mov  %rdx,%rbp

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#4)
# asm 2: mulq  0(<qp=%rcx)
mulq  0(%rcx)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#8
# asm 2: add  <mulrax=%rax,<a1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#4)
# asm 2: mulq  8(<qp=%rcx)
mulq  8(%rcx)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#10
# asm 2: add  <mulrax=%rax,<a2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#4)
# asm 2: mulq  16(<qp=%rcx)
mulq  16(%rcx)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#12
# asm 2: add  <mulrax=%rax,<a3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#4)
# asm 2: mulq  24(<qp=%rcx)
mulq  24(%rcx)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#14
# asm 2: add  <mulrax=%rax,<a4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = a1_stack
# asm 1: movq <a1_stack=stack64#9,>mulrax=int64#3
# asm 2: movq <a1_stack=64(%rsp),>mulrax=%rdx
movq 64(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#4)
# asm 2: mulq  32(<qp=%rcx)
mulq  32(%rcx)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#5
# asm 2: add  <mulrax=%rax,<a0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#4)
# asm 2: mulq  0(<qp=%rcx)
mulq  0(%rcx)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#10
# asm 2: add  <mulrax=%rax,<a2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#4)
# asm 2: mulq  8(<qp=%rcx)
mulq  8(%rcx)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#12
# asm 2: add  <mulrax=%rax,<a3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#7
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rax
movq 72(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#4)
# asm 2: mulq  16(<qp=%rcx)
mulq  16(%rcx)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#14
# asm 2: add  <mulrax=%rax,<a4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#3
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rdx
movq 72(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#4)
# asm 2: mulq  24(<qp=%rcx)
mulq  24(%rcx)

# qhasm:   carry? a0 += mulrax
# asm 1: add  <mulrax=int64#7,<a0=int64#5
# asm 2: add  <mulrax=%rax,<a0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = a2_stack
# asm 1: movq <a2_stack=stack64#10,>mulrax=int64#3
# asm 2: movq <a2_stack=72(%rsp),>mulrax=%rdx
movq 72(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#4)
# asm 2: mulq  32(<qp=%rcx)
mulq  32(%rcx)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#8
# asm 2: add  <mulrax=%rax,<a1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = a3_stack
# asm 1: movq <a3_stack=stack64#11,>mulrax=int64#7
# asm 2: movq <a3_stack=80(%rsp),>mulrax=%rax
movq 80(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#4)
# asm 2: mulq  0(<qp=%rcx)
mulq  0(%rcx)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#12
# asm 2: add  <mulrax=%rax,<a3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = a3_stack
# asm 1: movq <a3_stack=stack64#11,>mulrax=int64#7
# asm 2: movq <a3_stack=80(%rsp),>mulrax=%rax
movq 80(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 8)
# asm 1: mulq  8(<qp=int64#4)
# asm 2: mulq  8(<qp=%rcx)
mulq  8(%rcx)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#14
# asm 2: add  <mulrax=%rax,<a4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#4)
# asm 2: mulq  24(<qp=%rcx)
mulq  24(%rcx)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#8
# asm 2: add  <mulrax=%rax,<a1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#4)
# asm 2: mulq  32(<qp=%rcx)
mulq  32(%rcx)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#10
# asm 2: add  <mulrax=%rax,<a2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = a4_stack
# asm 1: movq <a4_stack=stack64#12,>mulrax=int64#7
# asm 2: movq <a4_stack=88(%rsp),>mulrax=%rax
movq 88(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 0)
# asm 1: mulq  0(<qp=int64#4)
# asm 2: mulq  0(<qp=%rcx)
mulq  0(%rcx)

# qhasm:   carry? a4 += mulrax
# asm 1: add  <mulrax=int64#7,<a4=int64#14
# asm 2: add  <mulrax=%rax,<a4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 16)
# asm 1: mulq  16(<qp=int64#4)
# asm 2: mulq  16(<qp=%rcx)
mulq  16(%rcx)

# qhasm:   carry? a1 += mulrax
# asm 1: add  <mulrax=int64#7,<a1=int64#8
# asm 2: add  <mulrax=%rax,<a1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 24)
# asm 1: mulq  24(<qp=int64#4)
# asm 2: mulq  24(<qp=%rcx)
mulq  24(%rcx)

# qhasm:   carry? a2 += mulrax
# asm 1: add  <mulrax=int64#7,<a2=int64#10
# asm 2: add  <mulrax=%rax,<a2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 32)
# asm 1: mulq  32(<qp=int64#4)
# asm 2: mulq  32(<qp=%rcx)
mulq  32(%rcx)

# qhasm:   carry? a3 += mulrax
# asm 1: add  <mulrax=int64#7,<a3=int64#12
# asm 2: add  <mulrax=%rax,<a3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.a0) << 13
# asm 1: shld $13,<a0=int64#5,<mulr01=int64#6
# asm 2: shld $13,<a0=%r8,<mulr01=%r9
shld $13,%r8,%r9

# qhasm:   a0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a0=int64#5
# asm 2: and  <mulredmask=%rdx,<a0=%r8
and  %rdx,%r8

# qhasm:   mulr11 = (mulr11.a1) << 13
# asm 1: shld $13,<a1=int64#8,<mulr11=int64#9
# asm 2: shld $13,<a1=%r10,<mulr11=%r11
shld $13,%r10,%r11

# qhasm:   a1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a1=int64#8
# asm 2: and  <mulredmask=%rdx,<a1=%r10
and  %rdx,%r10

# qhasm:   a1 += mulr01
# asm 1: add  <mulr01=int64#6,<a1=int64#8
# asm 2: add  <mulr01=%r9,<a1=%r10
add  %r9,%r10

# qhasm:   mulr21 = (mulr21.a2) << 13
# asm 1: shld $13,<a2=int64#10,<mulr21=int64#11
# asm 2: shld $13,<a2=%r12,<mulr21=%r13
shld $13,%r12,%r13

# qhasm:   a2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a2=int64#10
# asm 2: and  <mulredmask=%rdx,<a2=%r12
and  %rdx,%r12

# qhasm:   a2 += mulr11
# asm 1: add  <mulr11=int64#9,<a2=int64#10
# asm 2: add  <mulr11=%r11,<a2=%r12
add  %r11,%r12

# qhasm:   mulr31 = (mulr31.a3) << 13
# asm 1: shld $13,<a3=int64#12,<mulr31=int64#13
# asm 2: shld $13,<a3=%r14,<mulr31=%r15
shld $13,%r14,%r15

# qhasm:   a3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a3=int64#12
# asm 2: and  <mulredmask=%rdx,<a3=%r14
and  %rdx,%r14

# qhasm:   a3 += mulr21
# asm 1: add  <mulr21=int64#11,<a3=int64#12
# asm 2: add  <mulr21=%r13,<a3=%r14
add  %r13,%r14

# qhasm:   mulr41 = (mulr41.a4) << 13
# asm 1: shld $13,<a4=int64#14,<mulr41=int64#15
# asm 2: shld $13,<a4=%rbx,<mulr41=%rbp
shld $13,%rbx,%rbp

# qhasm:   a4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a4=int64#14
# asm 2: and  <mulredmask=%rdx,<a4=%rbx
and  %rdx,%rbx

# qhasm:   a4 += mulr31
# asm 1: add  <mulr31=int64#13,<a4=int64#14
# asm 2: add  <mulr31=%r15,<a4=%rbx
add  %r15,%rbx

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#15,>mulr41=int64#6
# asm 2: imulq  $19,<mulr41=%rbp,>mulr41=%r9
imulq  $19,%rbp,%r9

# qhasm:   a0 += mulr41
# asm 1: add  <mulr41=int64#6,<a0=int64#5
# asm 2: add  <mulr41=%r9,<a0=%r8
add  %r9,%r8

# qhasm:   mult = a0
# asm 1: mov  <a0=int64#5,>mult=int64#6
# asm 2: mov  <a0=%r8,>mult=%r9
mov  %r8,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   mult += a1
# asm 1: add  <a1=int64#8,<mult=int64#6
# asm 2: add  <a1=%r10,<mult=%r9
add  %r10,%r9

# qhasm:   a1 = mult
# asm 1: mov  <mult=int64#6,>a1=int64#7
# asm 2: mov  <mult=%r9,>a1=%rax
mov  %r9,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   a0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a0=int64#5
# asm 2: and  <mulredmask=%rdx,<a0=%r8
and  %rdx,%r8

# qhasm:   mult += a2
# asm 1: add  <a2=int64#10,<mult=int64#6
# asm 2: add  <a2=%r12,<mult=%r9
add  %r12,%r9

# qhasm:   a2 = mult
# asm 1: mov  <mult=int64#6,>a2=int64#8
# asm 2: mov  <mult=%r9,>a2=%r10
mov  %r9,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   a1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a1=int64#7
# asm 2: and  <mulredmask=%rdx,<a1=%rax
and  %rdx,%rax

# qhasm:   mult += a3
# asm 1: add  <a3=int64#12,<mult=int64#6
# asm 2: add  <a3=%r14,<mult=%r9
add  %r14,%r9

# qhasm:   a3 = mult
# asm 1: mov  <mult=int64#6,>a3=int64#9
# asm 2: mov  <mult=%r9,>a3=%r11
mov  %r9,%r11

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   a2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a2=int64#8
# asm 2: and  <mulredmask=%rdx,<a2=%r10
and  %rdx,%r10

# qhasm:   mult += a4
# asm 1: add  <a4=int64#14,<mult=int64#6
# asm 2: add  <a4=%rbx,<mult=%r9
add  %rbx,%r9

# qhasm:   a4 = mult
# asm 1: mov  <mult=int64#6,>a4=int64#10
# asm 2: mov  <mult=%r9,>a4=%r12
mov  %r9,%r12

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   a3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a3=int64#9
# asm 2: and  <mulredmask=%rdx,<a3=%r11
and  %rdx,%r11

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#6,>mult=int64#6
# asm 2: imulq  $19,<mult=%r9,>mult=%r9
imulq  $19,%r9,%r9

# qhasm:   a0 += mult
# asm 1: add  <mult=int64#6,<a0=int64#5
# asm 2: add  <mult=%r9,<a0=%r8
add  %r9,%r8

# qhasm:   a4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<a4=int64#10
# asm 2: and  <mulredmask=%rdx,<a4=%r12
and  %rdx,%r12

# qhasm: a0_stack = a0
# asm 1: movq <a0=int64#5,>a0_stack=stack64#8
# asm 2: movq <a0=%r8,>a0_stack=56(%rsp)
movq %r8,56(%rsp)

# qhasm: a1_stack = a1
# asm 1: movq <a1=int64#7,>a1_stack=stack64#9
# asm 2: movq <a1=%rax,>a1_stack=64(%rsp)
movq %rax,64(%rsp)

# qhasm: a2_stack = a2
# asm 1: movq <a2=int64#8,>a2_stack=stack64#10
# asm 2: movq <a2=%r10,>a2_stack=72(%rsp)
movq %r10,72(%rsp)

# qhasm: a3_stack = a3
# asm 1: movq <a3=int64#9,>a3_stack=stack64#11
# asm 2: movq <a3=%r11,>a3_stack=80(%rsp)
movq %r11,80(%rsp)

# qhasm: a4_stack = a4
# asm 1: movq <a4=int64#10,>a4_stack=stack64#12
# asm 2: movq <a4=%r12,>a4_stack=88(%rsp)
movq %r12,88(%rsp)

# qhasm:   mulrax = b3_stack
# asm 1: movq <b3_stack=stack64#16,>mulrax=int64#3
# asm 2: movq <b3_stack=120(%rsp),>mulrax=%rdx
movq 120(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#18
# asm 2: movq <mulrax=%rax,>mulx319_stack=136(%rsp)
movq %rax,136(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#4)
# asm 2: mulq  56(<qp=%rcx)
mulq  56(%rcx)

# qhasm:   e0 = mulrax
# asm 1: mov  <mulrax=int64#7,>e0=int64#5
# asm 2: mov  <mulrax=%rax,>e0=%r8
mov  %rax,%r8

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#6
# asm 2: mov  <mulrdx=%rdx,>mulr01=%r9
mov  %rdx,%r9

# qhasm:   mulrax = b4_stack
# asm 1: movq <b4_stack=stack64#17,>mulrax=int64#3
# asm 2: movq <b4_stack=128(%rsp),>mulrax=%rdx
movq 128(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#19
# asm 2: movq <mulrax=%rax,>mulx419_stack=144(%rsp)
movq %rax,144(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#4)
# asm 2: mulq  48(<qp=%rcx)
mulq  48(%rcx)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#5
# asm 2: add  <mulrax=%rax,<e0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#4)
# asm 2: mulq  40(<qp=%rcx)
mulq  40(%rcx)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#5
# asm 2: add  <mulrax=%rax,<e0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#4)
# asm 2: mulq  48(<qp=%rcx)
mulq  48(%rcx)

# qhasm:   e1 = mulrax
# asm 1: mov  <mulrax=int64#7,>e1=int64#8
# asm 2: mov  <mulrax=%rax,>e1=%r10
mov  %rax,%r10

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#9
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r11
mov  %rdx,%r11

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#4)
# asm 2: mulq  56(<qp=%rcx)
mulq  56(%rcx)

# qhasm:   e2 = mulrax
# asm 1: mov  <mulrax=int64#7,>e2=int64#10
# asm 2: mov  <mulrax=%rax,>e2=%r12
mov  %rax,%r12

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#11
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r13
mov  %rdx,%r13

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#4)
# asm 2: mulq  64(<qp=%rcx)
mulq  64(%rcx)

# qhasm:   e3 = mulrax
# asm 1: mov  <mulrax=int64#7,>e3=int64#12
# asm 2: mov  <mulrax=%rax,>e3=%r14
mov  %rax,%r14

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#13
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r15
mov  %rdx,%r15

# qhasm:   mulrax = b0_stack
# asm 1: movq <b0_stack=stack64#13,>mulrax=int64#7
# asm 2: movq <b0_stack=96(%rsp),>mulrax=%rax
movq 96(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#4)
# asm 2: mulq  72(<qp=%rcx)
mulq  72(%rcx)

# qhasm:   e4 = mulrax
# asm 1: mov  <mulrax=int64#7,>e4=int64#14
# asm 2: mov  <mulrax=%rax,>e4=%rbx
mov  %rax,%rbx

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#15
# asm 2: mov  <mulrdx=%rdx,>mulr41=%rbp
mov  %rdx,%rbp

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#4)
# asm 2: mulq  40(<qp=%rcx)
mulq  40(%rcx)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#8
# asm 2: add  <mulrax=%rax,<e1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#4)
# asm 2: mulq  48(<qp=%rcx)
mulq  48(%rcx)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#10
# asm 2: add  <mulrax=%rax,<e2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#4)
# asm 2: mulq  56(<qp=%rcx)
mulq  56(%rcx)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#12
# asm 2: add  <mulrax=%rax,<e3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#7
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rax
movq 104(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#4)
# asm 2: mulq  64(<qp=%rcx)
mulq  64(%rcx)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#14
# asm 2: add  <mulrax=%rax,<e4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = b1_stack
# asm 1: movq <b1_stack=stack64#14,>mulrax=int64#3
# asm 2: movq <b1_stack=104(%rsp),>mulrax=%rdx
movq 104(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#4)
# asm 2: mulq  72(<qp=%rcx)
mulq  72(%rcx)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#5
# asm 2: add  <mulrax=%rax,<e0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#4)
# asm 2: mulq  40(<qp=%rcx)
mulq  40(%rcx)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#10
# asm 2: add  <mulrax=%rax,<e2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#4)
# asm 2: mulq  48(<qp=%rcx)
mulq  48(%rcx)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#12
# asm 2: add  <mulrax=%rax,<e3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#7
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rax
movq 112(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#4)
# asm 2: mulq  56(<qp=%rcx)
mulq  56(%rcx)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#14
# asm 2: add  <mulrax=%rax,<e4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#4)
# asm 2: mulq  64(<qp=%rcx)
mulq  64(%rcx)

# qhasm:   carry? e0 += mulrax
# asm 1: add  <mulrax=int64#7,<e0=int64#5
# asm 2: add  <mulrax=%rax,<e0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = b2_stack
# asm 1: movq <b2_stack=stack64#15,>mulrax=int64#3
# asm 2: movq <b2_stack=112(%rsp),>mulrax=%rdx
movq 112(%rsp),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#4)
# asm 2: mulq  72(<qp=%rcx)
mulq  72(%rcx)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#8
# asm 2: add  <mulrax=%rax,<e1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = b3_stack
# asm 1: movq <b3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <b3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#4)
# asm 2: mulq  40(<qp=%rcx)
mulq  40(%rcx)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#12
# asm 2: add  <mulrax=%rax,<e3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = b3_stack
# asm 1: movq <b3_stack=stack64#16,>mulrax=int64#7
# asm 2: movq <b3_stack=120(%rsp),>mulrax=%rax
movq 120(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 48)
# asm 1: mulq  48(<qp=int64#4)
# asm 2: mulq  48(<qp=%rcx)
mulq  48(%rcx)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#14
# asm 2: add  <mulrax=%rax,<e4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#4)
# asm 2: mulq  64(<qp=%rcx)
mulq  64(%rcx)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#8
# asm 2: add  <mulrax=%rax,<e1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#18,>mulrax=int64#7
# asm 2: movq <mulx319_stack=136(%rsp),>mulrax=%rax
movq 136(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#4)
# asm 2: mulq  72(<qp=%rcx)
mulq  72(%rcx)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#10
# asm 2: add  <mulrax=%rax,<e2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = b4_stack
# asm 1: movq <b4_stack=stack64#17,>mulrax=int64#7
# asm 2: movq <b4_stack=128(%rsp),>mulrax=%rax
movq 128(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 40)
# asm 1: mulq  40(<qp=int64#4)
# asm 2: mulq  40(<qp=%rcx)
mulq  40(%rcx)

# qhasm:   carry? e4 += mulrax
# asm 1: add  <mulrax=int64#7,<e4=int64#14
# asm 2: add  <mulrax=%rax,<e4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 56)
# asm 1: mulq  56(<qp=int64#4)
# asm 2: mulq  56(<qp=%rcx)
mulq  56(%rcx)

# qhasm:   carry? e1 += mulrax
# asm 1: add  <mulrax=int64#7,<e1=int64#8
# asm 2: add  <mulrax=%rax,<e1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 64)
# asm 1: mulq  64(<qp=int64#4)
# asm 2: mulq  64(<qp=%rcx)
mulq  64(%rcx)

# qhasm:   carry? e2 += mulrax
# asm 1: add  <mulrax=int64#7,<e2=int64#10
# asm 2: add  <mulrax=%rax,<e2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#19,>mulrax=int64#7
# asm 2: movq <mulx419_stack=144(%rsp),>mulrax=%rax
movq 144(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 72)
# asm 1: mulq  72(<qp=int64#4)
# asm 2: mulq  72(<qp=%rcx)
mulq  72(%rcx)

# qhasm:   carry? e3 += mulrax
# asm 1: add  <mulrax=int64#7,<e3=int64#12
# asm 2: add  <mulrax=%rax,<e3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.e0) << 13
# asm 1: shld $13,<e0=int64#5,<mulr01=int64#6
# asm 2: shld $13,<e0=%r8,<mulr01=%r9
shld $13,%r8,%r9

# qhasm:   e0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e0=int64#5
# asm 2: and  <mulredmask=%rdx,<e0=%r8
and  %rdx,%r8

# qhasm:   mulr11 = (mulr11.e1) << 13
# asm 1: shld $13,<e1=int64#8,<mulr11=int64#9
# asm 2: shld $13,<e1=%r10,<mulr11=%r11
shld $13,%r10,%r11

# qhasm:   e1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e1=int64#8
# asm 2: and  <mulredmask=%rdx,<e1=%r10
and  %rdx,%r10

# qhasm:   e1 += mulr01
# asm 1: add  <mulr01=int64#6,<e1=int64#8
# asm 2: add  <mulr01=%r9,<e1=%r10
add  %r9,%r10

# qhasm:   mulr21 = (mulr21.e2) << 13
# asm 1: shld $13,<e2=int64#10,<mulr21=int64#11
# asm 2: shld $13,<e2=%r12,<mulr21=%r13
shld $13,%r12,%r13

# qhasm:   e2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e2=int64#10
# asm 2: and  <mulredmask=%rdx,<e2=%r12
and  %rdx,%r12

# qhasm:   e2 += mulr11
# asm 1: add  <mulr11=int64#9,<e2=int64#10
# asm 2: add  <mulr11=%r11,<e2=%r12
add  %r11,%r12

# qhasm:   mulr31 = (mulr31.e3) << 13
# asm 1: shld $13,<e3=int64#12,<mulr31=int64#13
# asm 2: shld $13,<e3=%r14,<mulr31=%r15
shld $13,%r14,%r15

# qhasm:   e3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e3=int64#12
# asm 2: and  <mulredmask=%rdx,<e3=%r14
and  %rdx,%r14

# qhasm:   e3 += mulr21
# asm 1: add  <mulr21=int64#11,<e3=int64#12
# asm 2: add  <mulr21=%r13,<e3=%r14
add  %r13,%r14

# qhasm:   mulr41 = (mulr41.e4) << 13
# asm 1: shld $13,<e4=int64#14,<mulr41=int64#15
# asm 2: shld $13,<e4=%rbx,<mulr41=%rbp
shld $13,%rbx,%rbp

# qhasm:   e4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e4=int64#14
# asm 2: and  <mulredmask=%rdx,<e4=%rbx
and  %rdx,%rbx

# qhasm:   e4 += mulr31
# asm 1: add  <mulr31=int64#13,<e4=int64#14
# asm 2: add  <mulr31=%r15,<e4=%rbx
add  %r15,%rbx

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#15,>mulr41=int64#6
# asm 2: imulq  $19,<mulr41=%rbp,>mulr41=%r9
imulq  $19,%rbp,%r9

# qhasm:   e0 += mulr41
# asm 1: add  <mulr41=int64#6,<e0=int64#5
# asm 2: add  <mulr41=%r9,<e0=%r8
add  %r9,%r8

# qhasm:   mult = e0
# asm 1: mov  <e0=int64#5,>mult=int64#6
# asm 2: mov  <e0=%r8,>mult=%r9
mov  %r8,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   mult += e1
# asm 1: add  <e1=int64#8,<mult=int64#6
# asm 2: add  <e1=%r10,<mult=%r9
add  %r10,%r9

# qhasm:   e1 = mult
# asm 1: mov  <mult=int64#6,>e1=int64#7
# asm 2: mov  <mult=%r9,>e1=%rax
mov  %r9,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   e0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e0=int64#5
# asm 2: and  <mulredmask=%rdx,<e0=%r8
and  %rdx,%r8

# qhasm:   mult += e2
# asm 1: add  <e2=int64#10,<mult=int64#6
# asm 2: add  <e2=%r12,<mult=%r9
add  %r12,%r9

# qhasm:   e2 = mult
# asm 1: mov  <mult=int64#6,>e2=int64#8
# asm 2: mov  <mult=%r9,>e2=%r10
mov  %r9,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   e1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e1=int64#7
# asm 2: and  <mulredmask=%rdx,<e1=%rax
and  %rdx,%rax

# qhasm:   mult += e3
# asm 1: add  <e3=int64#12,<mult=int64#6
# asm 2: add  <e3=%r14,<mult=%r9
add  %r14,%r9

# qhasm:   e3 = mult
# asm 1: mov  <mult=int64#6,>e3=int64#9
# asm 2: mov  <mult=%r9,>e3=%r11
mov  %r9,%r11

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   e2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e2=int64#8
# asm 2: and  <mulredmask=%rdx,<e2=%r10
and  %rdx,%r10

# qhasm:   mult += e4
# asm 1: add  <e4=int64#14,<mult=int64#6
# asm 2: add  <e4=%rbx,<mult=%r9
add  %rbx,%r9

# qhasm:   e4 = mult
# asm 1: mov  <mult=int64#6,>e4=int64#10
# asm 2: mov  <mult=%r9,>e4=%r12
mov  %r9,%r12

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#6
# asm 2: shr  $51,<mult=%r9
shr  $51,%r9

# qhasm:   e3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e3=int64#9
# asm 2: and  <mulredmask=%rdx,<e3=%r11
and  %rdx,%r11

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#6,>mult=int64#6
# asm 2: imulq  $19,<mult=%r9,>mult=%r9
imulq  $19,%r9,%r9

# qhasm:   e0 += mult
# asm 1: add  <mult=int64#6,<e0=int64#5
# asm 2: add  <mult=%r9,<e0=%r8
add  %r9,%r8

# qhasm:   e4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<e4=int64#10
# asm 2: and  <mulredmask=%rdx,<e4=%r12
and  %rdx,%r12

# qhasm: h0 = e0
# asm 1: mov  <e0=int64#5,>h0=int64#3
# asm 2: mov  <e0=%r8,>h0=%rdx
mov  %r8,%rdx

# qhasm: h1 = e1
# asm 1: mov  <e1=int64#7,>h1=int64#6
# asm 2: mov  <e1=%rax,>h1=%r9
mov  %rax,%r9

# qhasm: h2 = e2
# asm 1: mov  <e2=int64#8,>h2=int64#11
# asm 2: mov  <e2=%r10,>h2=%r13
mov  %r10,%r13

# qhasm: h3 = e3
# asm 1: mov  <e3=int64#9,>h3=int64#12
# asm 2: mov  <e3=%r11,>h3=%r14
mov  %r11,%r14

# qhasm: h4 = e4
# asm 1: mov  <e4=int64#10,>h4=int64#13
# asm 2: mov  <e4=%r12,>h4=%r15
mov  %r12,%r15

# qhasm: e0 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P0
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<e0=int64#5
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<e0=%r8
add  crypto_sign_ed25519_amd64_51_30k_batch_2P0(%rip),%r8

# qhasm: e1 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e1=int64#7
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e1=%rax
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rax

# qhasm: e2 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e2=int64#8
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e2=%r10
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r10

# qhasm: e3 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e3=int64#9
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e3=%r11
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r11

# qhasm: e4 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e4=int64#10
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<e4=%r12
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r12

# qhasm: h0 += a0_stack
# asm 1: addq <a0_stack=stack64#8,<h0=int64#3
# asm 2: addq <a0_stack=56(%rsp),<h0=%rdx
addq 56(%rsp),%rdx

# qhasm: h1 += a1_stack
# asm 1: addq <a1_stack=stack64#9,<h1=int64#6
# asm 2: addq <a1_stack=64(%rsp),<h1=%r9
addq 64(%rsp),%r9

# qhasm: h2 += a2_stack
# asm 1: addq <a2_stack=stack64#10,<h2=int64#11
# asm 2: addq <a2_stack=72(%rsp),<h2=%r13
addq 72(%rsp),%r13

# qhasm: h3 += a3_stack
# asm 1: addq <a3_stack=stack64#11,<h3=int64#12
# asm 2: addq <a3_stack=80(%rsp),<h3=%r14
addq 80(%rsp),%r14

# qhasm: h4 += a4_stack
# asm 1: addq <a4_stack=stack64#12,<h4=int64#13
# asm 2: addq <a4_stack=88(%rsp),<h4=%r15
addq 88(%rsp),%r15

# qhasm: e0 -= a0_stack
# asm 1: subq <a0_stack=stack64#8,<e0=int64#5
# asm 2: subq <a0_stack=56(%rsp),<e0=%r8
subq 56(%rsp),%r8

# qhasm: e1 -= a1_stack
# asm 1: subq <a1_stack=stack64#9,<e1=int64#7
# asm 2: subq <a1_stack=64(%rsp),<e1=%rax
subq 64(%rsp),%rax

# qhasm: e2 -= a2_stack
# asm 1: subq <a2_stack=stack64#10,<e2=int64#8
# asm 2: subq <a2_stack=72(%rsp),<e2=%r10
subq 72(%rsp),%r10

# qhasm: e3 -= a3_stack
# asm 1: subq <a3_stack=stack64#11,<e3=int64#9
# asm 2: subq <a3_stack=80(%rsp),<e3=%r11
subq 80(%rsp),%r11

# qhasm: e4 -= a4_stack
# asm 1: subq <a4_stack=stack64#12,<e4=int64#10
# asm 2: subq <a4_stack=88(%rsp),<e4=%r12
subq 88(%rsp),%r12

# qhasm: *(uint64 *)(rp + 80) = h0
# asm 1: movq   <h0=int64#3,80(<rp=int64#1)
# asm 2: movq   <h0=%rdx,80(<rp=%rdi)
movq   %rdx,80(%rdi)

# qhasm: *(uint64 *)(rp + 88) = h1
# asm 1: movq   <h1=int64#6,88(<rp=int64#1)
# asm 2: movq   <h1=%r9,88(<rp=%rdi)
movq   %r9,88(%rdi)

# qhasm: *(uint64 *)(rp + 96) = h2
# asm 1: movq   <h2=int64#11,96(<rp=int64#1)
# asm 2: movq   <h2=%r13,96(<rp=%rdi)
movq   %r13,96(%rdi)

# qhasm: *(uint64 *)(rp + 104) = h3
# asm 1: movq   <h3=int64#12,104(<rp=int64#1)
# asm 2: movq   <h3=%r14,104(<rp=%rdi)
movq   %r14,104(%rdi)

# qhasm: *(uint64 *)(rp + 112) = h4
# asm 1: movq   <h4=int64#13,112(<rp=int64#1)
# asm 2: movq   <h4=%r15,112(<rp=%rdi)
movq   %r15,112(%rdi)

# qhasm: *(uint64 *)(rp + 0) = e0
# asm 1: movq   <e0=int64#5,0(<rp=int64#1)
# asm 2: movq   <e0=%r8,0(<rp=%rdi)
movq   %r8,0(%rdi)

# qhasm: *(uint64 *)(rp + 8) = e1
# asm 1: movq   <e1=int64#7,8(<rp=int64#1)
# asm 2: movq   <e1=%rax,8(<rp=%rdi)
movq   %rax,8(%rdi)

# qhasm: *(uint64 *)(rp + 16) = e2
# asm 1: movq   <e2=int64#8,16(<rp=int64#1)
# asm 2: movq   <e2=%r10,16(<rp=%rdi)
movq   %r10,16(%rdi)

# qhasm: *(uint64 *)(rp + 24) = e3
# asm 1: movq   <e3=int64#9,24(<rp=int64#1)
# asm 2: movq   <e3=%r11,24(<rp=%rdi)
movq   %r11,24(%rdi)

# qhasm: *(uint64 *)(rp + 32) = e4
# asm 1: movq   <e4=int64#10,32(<rp=int64#1)
# asm 2: movq   <e4=%r12,32(<rp=%rdi)
movq   %r12,32(%rdi)

# qhasm:   mulrax = *(uint64 *)(pp + 144)
# asm 1: movq   144(<pp=int64#2),>mulrax=int64#3
# asm 2: movq   144(<pp=%rsi),>mulrax=%rdx
movq   144(%rsi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx319_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx319_stack=stack64#8
# asm 2: movq <mulrax=%rax,>mulx319_stack=56(%rsp)
movq %rax,56(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#4)
# asm 2: mulq  96(<qp=%rcx)
mulq  96(%rcx)

# qhasm:   c0 = mulrax
# asm 1: mov  <mulrax=int64#7,>c0=int64#5
# asm 2: mov  <mulrax=%rax,>c0=%r8
mov  %rax,%r8

# qhasm:   mulr01 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr01=int64#6
# asm 2: mov  <mulrdx=%rdx,>mulr01=%r9
mov  %rdx,%r9

# qhasm:   mulrax = *(uint64 *)(pp + 152)
# asm 1: movq   152(<pp=int64#2),>mulrax=int64#3
# asm 2: movq   152(<pp=%rsi),>mulrax=%rdx
movq   152(%rsi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   mulx419_stack = mulrax
# asm 1: movq <mulrax=int64#7,>mulx419_stack=stack64#9
# asm 2: movq <mulrax=%rax,>mulx419_stack=64(%rsp)
movq %rax,64(%rsp)

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#4)
# asm 2: mulq  88(<qp=%rcx)
mulq  88(%rcx)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#5
# asm 2: add  <mulrax=%rax,<c0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = *(uint64 *)(pp + 120)
# asm 1: movq   120(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   120(<pp=%rsi),>mulrax=%rax
movq   120(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#4)
# asm 2: mulq  80(<qp=%rcx)
mulq  80(%rcx)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#5
# asm 2: add  <mulrax=%rax,<c0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = *(uint64 *)(pp + 120)
# asm 1: movq   120(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   120(<pp=%rsi),>mulrax=%rax
movq   120(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#4)
# asm 2: mulq  88(<qp=%rcx)
mulq  88(%rcx)

# qhasm:   c1 = mulrax
# asm 1: mov  <mulrax=int64#7,>c1=int64#8
# asm 2: mov  <mulrax=%rax,>c1=%r10
mov  %rax,%r10

# qhasm:   mulr11 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr11=int64#9
# asm 2: mov  <mulrdx=%rdx,>mulr11=%r11
mov  %rdx,%r11

# qhasm:   mulrax = *(uint64 *)(pp + 120)
# asm 1: movq   120(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   120(<pp=%rsi),>mulrax=%rax
movq   120(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#4)
# asm 2: mulq  96(<qp=%rcx)
mulq  96(%rcx)

# qhasm:   c2 = mulrax
# asm 1: mov  <mulrax=int64#7,>c2=int64#10
# asm 2: mov  <mulrax=%rax,>c2=%r12
mov  %rax,%r12

# qhasm:   mulr21 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr21=int64#11
# asm 2: mov  <mulrdx=%rdx,>mulr21=%r13
mov  %rdx,%r13

# qhasm:   mulrax = *(uint64 *)(pp + 120)
# asm 1: movq   120(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   120(<pp=%rsi),>mulrax=%rax
movq   120(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#4)
# asm 2: mulq  104(<qp=%rcx)
mulq  104(%rcx)

# qhasm:   c3 = mulrax
# asm 1: mov  <mulrax=int64#7,>c3=int64#12
# asm 2: mov  <mulrax=%rax,>c3=%r14
mov  %rax,%r14

# qhasm:   mulr31 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr31=int64#13
# asm 2: mov  <mulrdx=%rdx,>mulr31=%r15
mov  %rdx,%r15

# qhasm:   mulrax = *(uint64 *)(pp + 120)
# asm 1: movq   120(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   120(<pp=%rsi),>mulrax=%rax
movq   120(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#4)
# asm 2: mulq  112(<qp=%rcx)
mulq  112(%rcx)

# qhasm:   c4 = mulrax
# asm 1: mov  <mulrax=int64#7,>c4=int64#14
# asm 2: mov  <mulrax=%rax,>c4=%rbx
mov  %rax,%rbx

# qhasm:   mulr41 = mulrdx
# asm 1: mov  <mulrdx=int64#3,>mulr41=int64#15
# asm 2: mov  <mulrdx=%rdx,>mulr41=%rbp
mov  %rdx,%rbp

# qhasm:   mulrax = *(uint64 *)(pp + 128)
# asm 1: movq   128(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   128(<pp=%rsi),>mulrax=%rax
movq   128(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#4)
# asm 2: mulq  80(<qp=%rcx)
mulq  80(%rcx)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#8
# asm 2: add  <mulrax=%rax,<c1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = *(uint64 *)(pp + 128)
# asm 1: movq   128(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   128(<pp=%rsi),>mulrax=%rax
movq   128(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#4)
# asm 2: mulq  88(<qp=%rcx)
mulq  88(%rcx)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#10
# asm 2: add  <mulrax=%rax,<c2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = *(uint64 *)(pp + 128)
# asm 1: movq   128(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   128(<pp=%rsi),>mulrax=%rax
movq   128(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#4)
# asm 2: mulq  96(<qp=%rcx)
mulq  96(%rcx)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#12
# asm 2: add  <mulrax=%rax,<c3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = *(uint64 *)(pp + 128)
# asm 1: movq   128(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   128(<pp=%rsi),>mulrax=%rax
movq   128(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#4)
# asm 2: mulq  104(<qp=%rcx)
mulq  104(%rcx)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#14
# asm 2: add  <mulrax=%rax,<c4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = *(uint64 *)(pp + 128)
# asm 1: movq   128(<pp=int64#2),>mulrax=int64#3
# asm 2: movq   128(<pp=%rsi),>mulrax=%rdx
movq   128(%rsi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#4)
# asm 2: mulq  112(<qp=%rcx)
mulq  112(%rcx)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#5
# asm 2: add  <mulrax=%rax,<c0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = *(uint64 *)(pp + 136)
# asm 1: movq   136(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   136(<pp=%rsi),>mulrax=%rax
movq   136(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#4)
# asm 2: mulq  80(<qp=%rcx)
mulq  80(%rcx)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#10
# asm 2: add  <mulrax=%rax,<c2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = *(uint64 *)(pp + 136)
# asm 1: movq   136(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   136(<pp=%rsi),>mulrax=%rax
movq   136(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#4)
# asm 2: mulq  88(<qp=%rcx)
mulq  88(%rcx)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#12
# asm 2: add  <mulrax=%rax,<c3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = *(uint64 *)(pp + 136)
# asm 1: movq   136(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   136(<pp=%rsi),>mulrax=%rax
movq   136(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#4)
# asm 2: mulq  96(<qp=%rcx)
mulq  96(%rcx)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#14
# asm 2: add  <mulrax=%rax,<c4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = *(uint64 *)(pp + 136)
# asm 1: movq   136(<pp=int64#2),>mulrax=int64#3
# asm 2: movq   136(<pp=%rsi),>mulrax=%rdx
movq   136(%rsi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#4)
# asm 2: mulq  104(<qp=%rcx)
mulq  104(%rcx)

# qhasm:   carry? c0 += mulrax
# asm 1: add  <mulrax=int64#7,<c0=int64#5
# asm 2: add  <mulrax=%rax,<c0=%r8
add  %rax,%r8

# qhasm:   mulr01 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr01=int64#6
# asm 2: adc <mulrdx=%rdx,<mulr01=%r9
adc %rdx,%r9

# qhasm:   mulrax = *(uint64 *)(pp + 136)
# asm 1: movq   136(<pp=int64#2),>mulrax=int64#3
# asm 2: movq   136(<pp=%rsi),>mulrax=%rdx
movq   136(%rsi),%rdx

# qhasm:   mulrax *= 19
# asm 1: imulq  $19,<mulrax=int64#3,>mulrax=int64#7
# asm 2: imulq  $19,<mulrax=%rdx,>mulrax=%rax
imulq  $19,%rdx,%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#4)
# asm 2: mulq  112(<qp=%rcx)
mulq  112(%rcx)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#8
# asm 2: add  <mulrax=%rax,<c1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = *(uint64 *)(pp + 144)
# asm 1: movq   144(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   144(<pp=%rsi),>mulrax=%rax
movq   144(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#4)
# asm 2: mulq  80(<qp=%rcx)
mulq  80(%rcx)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#12
# asm 2: add  <mulrax=%rax,<c3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulrax = *(uint64 *)(pp + 144)
# asm 1: movq   144(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   144(<pp=%rsi),>mulrax=%rax
movq   144(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 88)
# asm 1: mulq  88(<qp=int64#4)
# asm 2: mulq  88(<qp=%rcx)
mulq  88(%rcx)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#14
# asm 2: add  <mulrax=%rax,<c4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <mulx319_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#4)
# asm 2: mulq  104(<qp=%rcx)
mulq  104(%rcx)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#8
# asm 2: add  <mulrax=%rax,<c1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx319_stack
# asm 1: movq <mulx319_stack=stack64#8,>mulrax=int64#7
# asm 2: movq <mulx319_stack=56(%rsp),>mulrax=%rax
movq 56(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#4)
# asm 2: mulq  112(<qp=%rcx)
mulq  112(%rcx)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#10
# asm 2: add  <mulrax=%rax,<c2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = *(uint64 *)(pp + 152)
# asm 1: movq   152(<pp=int64#2),>mulrax=int64#7
# asm 2: movq   152(<pp=%rsi),>mulrax=%rax
movq   152(%rsi),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 80)
# asm 1: mulq  80(<qp=int64#4)
# asm 2: mulq  80(<qp=%rcx)
mulq  80(%rcx)

# qhasm:   carry? c4 += mulrax
# asm 1: add  <mulrax=int64#7,<c4=int64#14
# asm 2: add  <mulrax=%rax,<c4=%rbx
add  %rax,%rbx

# qhasm:   mulr41 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr41=int64#15
# asm 2: adc <mulrdx=%rdx,<mulr41=%rbp
adc %rdx,%rbp

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <mulx419_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 96)
# asm 1: mulq  96(<qp=int64#4)
# asm 2: mulq  96(<qp=%rcx)
mulq  96(%rcx)

# qhasm:   carry? c1 += mulrax
# asm 1: add  <mulrax=int64#7,<c1=int64#8
# asm 2: add  <mulrax=%rax,<c1=%r10
add  %rax,%r10

# qhasm:   mulr11 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr11=int64#9
# asm 2: adc <mulrdx=%rdx,<mulr11=%r11
adc %rdx,%r11

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <mulx419_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 104)
# asm 1: mulq  104(<qp=int64#4)
# asm 2: mulq  104(<qp=%rcx)
mulq  104(%rcx)

# qhasm:   carry? c2 += mulrax
# asm 1: add  <mulrax=int64#7,<c2=int64#10
# asm 2: add  <mulrax=%rax,<c2=%r12
add  %rax,%r12

# qhasm:   mulr21 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr21=int64#11
# asm 2: adc <mulrdx=%rdx,<mulr21=%r13
adc %rdx,%r13

# qhasm:   mulrax = mulx419_stack
# asm 1: movq <mulx419_stack=stack64#9,>mulrax=int64#7
# asm 2: movq <mulx419_stack=64(%rsp),>mulrax=%rax
movq 64(%rsp),%rax

# qhasm:   (uint128) mulrdx mulrax = mulrax * *(uint64 *)(qp + 112)
# asm 1: mulq  112(<qp=int64#4)
# asm 2: mulq  112(<qp=%rcx)
mulq  112(%rcx)

# qhasm:   carry? c3 += mulrax
# asm 1: add  <mulrax=int64#7,<c3=int64#12
# asm 2: add  <mulrax=%rax,<c3=%r14
add  %rax,%r14

# qhasm:   mulr31 += mulrdx + carry
# asm 1: adc <mulrdx=int64#3,<mulr31=int64#13
# asm 2: adc <mulrdx=%rdx,<mulr31=%r15
adc %rdx,%r15

# qhasm:   mulredmask = *(uint64 *) &crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51
# asm 1: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=int64#3
# asm 2: movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51,>mulredmask=%rdx
movq crypto_sign_ed25519_amd64_51_30k_batch_REDMASK51(%rip),%rdx

# qhasm:   mulr01 = (mulr01.c0) << 13
# asm 1: shld $13,<c0=int64#5,<mulr01=int64#6
# asm 2: shld $13,<c0=%r8,<mulr01=%r9
shld $13,%r8,%r9

# qhasm:   c0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c0=int64#5
# asm 2: and  <mulredmask=%rdx,<c0=%r8
and  %rdx,%r8

# qhasm:   mulr11 = (mulr11.c1) << 13
# asm 1: shld $13,<c1=int64#8,<mulr11=int64#9
# asm 2: shld $13,<c1=%r10,<mulr11=%r11
shld $13,%r10,%r11

# qhasm:   c1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c1=int64#8
# asm 2: and  <mulredmask=%rdx,<c1=%r10
and  %rdx,%r10

# qhasm:   c1 += mulr01
# asm 1: add  <mulr01=int64#6,<c1=int64#8
# asm 2: add  <mulr01=%r9,<c1=%r10
add  %r9,%r10

# qhasm:   mulr21 = (mulr21.c2) << 13
# asm 1: shld $13,<c2=int64#10,<mulr21=int64#11
# asm 2: shld $13,<c2=%r12,<mulr21=%r13
shld $13,%r12,%r13

# qhasm:   c2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c2=int64#10
# asm 2: and  <mulredmask=%rdx,<c2=%r12
and  %rdx,%r12

# qhasm:   c2 += mulr11
# asm 1: add  <mulr11=int64#9,<c2=int64#10
# asm 2: add  <mulr11=%r11,<c2=%r12
add  %r11,%r12

# qhasm:   mulr31 = (mulr31.c3) << 13
# asm 1: shld $13,<c3=int64#12,<mulr31=int64#13
# asm 2: shld $13,<c3=%r14,<mulr31=%r15
shld $13,%r14,%r15

# qhasm:   c3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c3=int64#12
# asm 2: and  <mulredmask=%rdx,<c3=%r14
and  %rdx,%r14

# qhasm:   c3 += mulr21
# asm 1: add  <mulr21=int64#11,<c3=int64#12
# asm 2: add  <mulr21=%r13,<c3=%r14
add  %r13,%r14

# qhasm:   mulr41 = (mulr41.c4) << 13
# asm 1: shld $13,<c4=int64#14,<mulr41=int64#15
# asm 2: shld $13,<c4=%rbx,<mulr41=%rbp
shld $13,%rbx,%rbp

# qhasm:   c4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c4=int64#14
# asm 2: and  <mulredmask=%rdx,<c4=%rbx
and  %rdx,%rbx

# qhasm:   c4 += mulr31
# asm 1: add  <mulr31=int64#13,<c4=int64#14
# asm 2: add  <mulr31=%r15,<c4=%rbx
add  %r15,%rbx

# qhasm:   mulr41 = mulr41 * 19
# asm 1: imulq  $19,<mulr41=int64#15,>mulr41=int64#4
# asm 2: imulq  $19,<mulr41=%rbp,>mulr41=%rcx
imulq  $19,%rbp,%rcx

# qhasm:   c0 += mulr41
# asm 1: add  <mulr41=int64#4,<c0=int64#5
# asm 2: add  <mulr41=%rcx,<c0=%r8
add  %rcx,%r8

# qhasm:   mult = c0
# asm 1: mov  <c0=int64#5,>mult=int64#4
# asm 2: mov  <c0=%r8,>mult=%rcx
mov  %r8,%rcx

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   mult += c1
# asm 1: add  <c1=int64#8,<mult=int64#4
# asm 2: add  <c1=%r10,<mult=%rcx
add  %r10,%rcx

# qhasm:   c1 = mult
# asm 1: mov  <mult=int64#4,>c1=int64#6
# asm 2: mov  <mult=%rcx,>c1=%r9
mov  %rcx,%r9

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   c0 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c0=int64#5
# asm 2: and  <mulredmask=%rdx,<c0=%r8
and  %rdx,%r8

# qhasm:   mult += c2
# asm 1: add  <c2=int64#10,<mult=int64#4
# asm 2: add  <c2=%r12,<mult=%rcx
add  %r12,%rcx

# qhasm:   c2 = mult
# asm 1: mov  <mult=int64#4,>c2=int64#7
# asm 2: mov  <mult=%rcx,>c2=%rax
mov  %rcx,%rax

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   c1 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c1=int64#6
# asm 2: and  <mulredmask=%rdx,<c1=%r9
and  %rdx,%r9

# qhasm:   mult += c3
# asm 1: add  <c3=int64#12,<mult=int64#4
# asm 2: add  <c3=%r14,<mult=%rcx
add  %r14,%rcx

# qhasm:   c3 = mult
# asm 1: mov  <mult=int64#4,>c3=int64#8
# asm 2: mov  <mult=%rcx,>c3=%r10
mov  %rcx,%r10

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   c2 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c2=int64#7
# asm 2: and  <mulredmask=%rdx,<c2=%rax
and  %rdx,%rax

# qhasm:   mult += c4
# asm 1: add  <c4=int64#14,<mult=int64#4
# asm 2: add  <c4=%rbx,<mult=%rcx
add  %rbx,%rcx

# qhasm:   c4 = mult
# asm 1: mov  <mult=int64#4,>c4=int64#9
# asm 2: mov  <mult=%rcx,>c4=%r11
mov  %rcx,%r11

# qhasm:   (uint64) mult >>= 51
# asm 1: shr  $51,<mult=int64#4
# asm 2: shr  $51,<mult=%rcx
shr  $51,%rcx

# qhasm:   c3 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c3=int64#8
# asm 2: and  <mulredmask=%rdx,<c3=%r10
and  %rdx,%r10

# qhasm:   mult *= 19
# asm 1: imulq  $19,<mult=int64#4,>mult=int64#4
# asm 2: imulq  $19,<mult=%rcx,>mult=%rcx
imulq  $19,%rcx,%rcx

# qhasm:   c0 += mult
# asm 1: add  <mult=int64#4,<c0=int64#5
# asm 2: add  <mult=%rcx,<c0=%r8
add  %rcx,%r8

# qhasm:   c4 &= mulredmask
# asm 1: and  <mulredmask=int64#3,<c4=int64#9
# asm 2: and  <mulredmask=%rdx,<c4=%r11
and  %rdx,%r11

# qhasm: c0_stack = c0
# asm 1: movq <c0=int64#5,>c0_stack=stack64#8
# asm 2: movq <c0=%r8,>c0_stack=56(%rsp)
movq %r8,56(%rsp)

# qhasm: f0 = *(uint64 *)(pp + 80)
# asm 1: movq   80(<pp=int64#2),>f0=int64#3
# asm 2: movq   80(<pp=%rsi),>f0=%rdx
movq   80(%rsi),%rdx

# qhasm: f1 = *(uint64 *)(pp + 88)
# asm 1: movq   88(<pp=int64#2),>f1=int64#4
# asm 2: movq   88(<pp=%rsi),>f1=%rcx
movq   88(%rsi),%rcx

# qhasm: f2 = *(uint64 *)(pp + 96)
# asm 1: movq   96(<pp=int64#2),>f2=int64#5
# asm 2: movq   96(<pp=%rsi),>f2=%r8
movq   96(%rsi),%r8

# qhasm: f3 = *(uint64 *)(pp + 104)
# asm 1: movq   104(<pp=int64#2),>f3=int64#10
# asm 2: movq   104(<pp=%rsi),>f3=%r12
movq   104(%rsi),%r12

# qhasm: f4 = *(uint64 *)(pp + 112)
# asm 1: movq   112(<pp=int64#2),>f4=int64#2
# asm 2: movq   112(<pp=%rsi),>f4=%rsi
movq   112(%rsi),%rsi

# qhasm: f0 += f0
# asm 1: add  <f0=int64#3,<f0=int64#3
# asm 2: add  <f0=%rdx,<f0=%rdx
add  %rdx,%rdx

# qhasm: f1 += f1
# asm 1: add  <f1=int64#4,<f1=int64#4
# asm 2: add  <f1=%rcx,<f1=%rcx
add  %rcx,%rcx

# qhasm: f2 += f2
# asm 1: add  <f2=int64#5,<f2=int64#5
# asm 2: add  <f2=%r8,<f2=%r8
add  %r8,%r8

# qhasm: f3 += f3
# asm 1: add  <f3=int64#10,<f3=int64#10
# asm 2: add  <f3=%r12,<f3=%r12
add  %r12,%r12

# qhasm: f4 += f4
# asm 1: add  <f4=int64#2,<f4=int64#2
# asm 2: add  <f4=%rsi,<f4=%rsi
add  %rsi,%rsi

# qhasm: g0 = f0
# asm 1: mov  <f0=int64#3,>g0=int64#11
# asm 2: mov  <f0=%rdx,>g0=%r13
mov  %rdx,%r13

# qhasm: g1 = f1
# asm 1: mov  <f1=int64#4,>g1=int64#12
# asm 2: mov  <f1=%rcx,>g1=%r14
mov  %rcx,%r14

# qhasm: g2 = f2
# asm 1: mov  <f2=int64#5,>g2=int64#13
# asm 2: mov  <f2=%r8,>g2=%r15
mov  %r8,%r15

# qhasm: g3 = f3
# asm 1: mov  <f3=int64#10,>g3=int64#14
# asm 2: mov  <f3=%r12,>g3=%rbx
mov  %r12,%rbx

# qhasm: g4 = f4
# asm 1: mov  <f4=int64#2,>g4=int64#15
# asm 2: mov  <f4=%rsi,>g4=%rbp
mov  %rsi,%rbp

# qhasm: f0 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P0
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<f0=int64#3
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P0,<f0=%rdx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P0(%rip),%rdx

# qhasm: f1 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f1=int64#4
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f1=%rcx
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rcx

# qhasm: f2 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f2=int64#5
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f2=%r8
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r8

# qhasm: f3 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f3=int64#10
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f3=%r12
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%r12

# qhasm: f4 += *(uint64 *)&crypto_sign_ed25519_amd64_51_30k_batch_2P1234
# asm 1: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f4=int64#2
# asm 2: add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234,<f4=%rsi
add  crypto_sign_ed25519_amd64_51_30k_batch_2P1234(%rip),%rsi

# qhasm: g0 += c0_stack
# asm 1: addq <c0_stack=stack64#8,<g0=int64#11
# asm 2: addq <c0_stack=56(%rsp),<g0=%r13
addq 56(%rsp),%r13

# qhasm: g1 += c1
# asm 1: add  <c1=int64#6,<g1=int64#12
# asm 2: add  <c1=%r9,<g1=%r14
add  %r9,%r14

# qhasm: g2 += c2
# asm 1: add  <c2=int64#7,<g2=int64#13
# asm 2: add  <c2=%rax,<g2=%r15
add  %rax,%r15

# qhasm: g3 += c3
# asm 1: add  <c3=int64#8,<g3=int64#14
# asm 2: add  <c3=%r10,<g3=%rbx
add  %r10,%rbx

# qhasm: g4 += c4
# asm 1: add  <c4=int64#9,<g4=int64#15
# asm 2: add  <c4=%r11,<g4=%rbp
add  %r11,%rbp

# qhasm: f0 -= c0_stack
# asm 1: subq <c0_stack=stack64#8,<f0=int64#3
# asm 2: subq <c0_stack=56(%rsp),<f0=%rdx
subq 56(%rsp),%rdx

# qhasm: f1 -= c1
# asm 1: sub  <c1=int64#6,<f1=int64#4
# asm 2: sub  <c1=%r9,<f1=%rcx
sub  %r9,%rcx

# qhasm: f2 -= c2
# asm 1: sub  <c2=int64#7,<f2=int64#5
# asm 2: sub  <c2=%rax,<f2=%r8
sub  %rax,%r8

# qhasm: f3 -= c3
# asm 1: sub  <c3=int64#8,<f3=int64#10
# asm 2: sub  <c3=%r10,<f3=%r12
sub  %r10,%r12

# qhasm: f4 -= c4
# asm 1: sub  <c4=int64#9,<f4=int64#2
# asm 2: sub  <c4=%r11,<f4=%rsi
sub  %r11,%rsi

# qhasm: *(uint64 *)(rp + 40) = g0
# asm 1: movq   <g0=int64#11,40(<rp=int64#1)
# asm 2: movq   <g0=%r13,40(<rp=%rdi)
movq   %r13,40(%rdi)

# qhasm: *(uint64 *)(rp + 48) = g1
# asm 1: movq   <g1=int64#12,48(<rp=int64#1)
# asm 2: movq   <g1=%r14,48(<rp=%rdi)
movq   %r14,48(%rdi)

# qhasm: *(uint64 *)(rp + 56) = g2
# asm 1: movq   <g2=int64#13,56(<rp=int64#1)
# asm 2: movq   <g2=%r15,56(<rp=%rdi)
movq   %r15,56(%rdi)

# qhasm: *(uint64 *)(rp + 64) = g3
# asm 1: movq   <g3=int64#14,64(<rp=int64#1)
# asm 2: movq   <g3=%rbx,64(<rp=%rdi)
movq   %rbx,64(%rdi)

# qhasm: *(uint64 *)(rp + 72) = g4
# asm 1: movq   <g4=int64#15,72(<rp=int64#1)
# asm 2: movq   <g4=%rbp,72(<rp=%rdi)
movq   %rbp,72(%rdi)

# qhasm: *(uint64 *)(rp + 120) = f0
# asm 1: movq   <f0=int64#3,120(<rp=int64#1)
# asm 2: movq   <f0=%rdx,120(<rp=%rdi)
movq   %rdx,120(%rdi)

# qhasm: *(uint64 *)(rp + 128) = f1
# asm 1: movq   <f1=int64#4,128(<rp=int64#1)
# asm 2: movq   <f1=%rcx,128(<rp=%rdi)
movq   %rcx,128(%rdi)

# qhasm: *(uint64 *)(rp + 136) = f2
# asm 1: movq   <f2=int64#5,136(<rp=int64#1)
# asm 2: movq   <f2=%r8,136(<rp=%rdi)
movq   %r8,136(%rdi)

# qhasm: *(uint64 *)(rp + 144) = f3
# asm 1: movq   <f3=int64#10,144(<rp=int64#1)
# asm 2: movq   <f3=%r12,144(<rp=%rdi)
movq   %r12,144(%rdi)

# qhasm: *(uint64 *)(rp + 152) = f4
# asm 1: movq   <f4=int64#2,152(<rp=int64#1)
# asm 2: movq   <f4=%rsi,152(<rp=%rdi)
movq   %rsi,152(%rdi)

# qhasm:   caller1 = caller1_stack
# asm 1: movq <caller1_stack=stack64#1,>caller1=int64#9
# asm 2: movq <caller1_stack=0(%rsp),>caller1=%r11
movq 0(%rsp),%r11

# qhasm:   caller2 = caller2_stack
# asm 1: movq <caller2_stack=stack64#2,>caller2=int64#10
# asm 2: movq <caller2_stack=8(%rsp),>caller2=%r12
movq 8(%rsp),%r12

# qhasm:   caller3 = caller3_stack
# asm 1: movq <caller3_stack=stack64#3,>caller3=int64#11
# asm 2: movq <caller3_stack=16(%rsp),>caller3=%r13
movq 16(%rsp),%r13

# qhasm:   caller4 = caller4_stack
# asm 1: movq <caller4_stack=stack64#4,>caller4=int64#12
# asm 2: movq <caller4_stack=24(%rsp),>caller4=%r14
movq 24(%rsp),%r14

# qhasm:   caller5 = caller5_stack
# asm 1: movq <caller5_stack=stack64#5,>caller5=int64#13
# asm 2: movq <caller5_stack=32(%rsp),>caller5=%r15
movq 32(%rsp),%r15

# qhasm:   caller6 = caller6_stack
# asm 1: movq <caller6_stack=stack64#6,>caller6=int64#14
# asm 2: movq <caller6_stack=40(%rsp),>caller6=%rbx
movq 40(%rsp),%rbx

# qhasm:   caller7 = caller7_stack
# asm 1: movq <caller7_stack=stack64#7,>caller7=int64#15
# asm 2: movq <caller7_stack=48(%rsp),>caller7=%rbp
movq 48(%rsp),%rbp

# qhasm: leave
add %r11,%rsp
mov %rdi,%rax
mov %rsi,%rdx
ret
